# Copyright: 2007-2013, Sebastian Billaudelle <sbillaudelle@googlemail.com>
#            2010-2013, Kristoffer Kleine <kris.kleine@yahoo.de>

# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2.1 of the License, or
# (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

def slugify(s):
    """
    Returns a slug-ready version of ``s``.
    (Lowercases all letters, replaces non-alphanumeric letters with hyphens,
    replaces all whitespace with underscores)
    """
    import re
    return re.sub('[^a-z0-9]', '-', re.sub('\s', '_', s.lower()))

def crop_string(s, maxlen, appendix='...'):
    """
    Crop string ``s`` after ``maxlen`` chars and append ``appendix``.
    """
    if len(s) > maxlen:
        return s[:maxlen] + appendix
    else:
        return s
